"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const data_table_1 = __importDefault(require("@cucumber/cucumber/lib/models/data_table"));
const tag_expressions_1 = __importDefault(require("@cucumber/tag-expressions"));
const assertions_1 = require("./assertions");
const registry_1 = __importDefault(require("./registry"));
const ast_helpers_1 = require("./ast-helpers");
function mapTagName(value) {
    return assertions_1.assertAndReturn(value.name, "Expected tag to have a name");
}
function createFeature(gherkinDocument, pickles, feature, testFilter) {
    describe(feature.name || "<unamed feature>", () => {
        if (feature.children) {
            for (const child of feature.children) {
                if (child.scenario) {
                    createScenario(gherkinDocument, pickles, child.scenario, testFilter);
                }
                else if (child.rule) {
                    createRule(gherkinDocument, pickles, child.rule, testFilter);
                }
            }
        }
    });
}
function createRule(gherkinDocument, pickles, rule, testFilter) {
    describe(rule.name || "<unamed rule>", () => {
        if (rule.children) {
            for (const child of rule.children) {
                if (child.scenario) {
                    createScenario(gherkinDocument, pickles, child.scenario, testFilter);
                }
            }
        }
    });
}
function createWeakCache(mapper) {
    return {
        cache: new WeakMap(),
        get(key) {
            const cacheHit = this.cache.get(key);
            if (cacheHit) {
                return cacheHit;
            }
            const value = mapper(key);
            this.cache.set(key, value);
            return value;
        },
    };
}
const gherkinDocumentsAstIdMaps = createWeakCache((key) => {
    const astIdMap = new Map();
    for (const node of ast_helpers_1.traverseGherkinDocument(key)) {
        if ("id" in node && node.id) {
            astIdMap.set(node.id, node);
        }
    }
    return astIdMap;
});
function createScenario(gherkinDocument, pickles, scenario, testFilter) {
    if (scenario.examples) {
        const exampleIds = scenario.examples
            .map((examples) => {
            return assertions_1.assertAndReturn(examples.tableBody, "Expected to find a table body").map((row) => assertions_1.assertAndReturn(row.id, "Expected table row to have an id"));
        })
            .reduce((acum, el) => acum.concat(el), []);
        const picklesToRun = exampleIds
            .map((exampleId) => assertions_1.assertAndReturn(pickles.find((pickle) => pickle.astNodeIds && pickle.astNodeIds.includes(exampleId)), `Expected to find a pickle associated with id = ${exampleId}`))
            .filter((pickle) => { var _a; return testFilter.evaluate(((_a = pickle.tags) === null || _a === void 0 ? void 0 : _a.map(mapTagName)) || []); });
        for (let i = 0; i < picklesToRun.length; i++) {
            const pickle = picklesToRun[i];
            const exampleName = `${assertions_1.assertAndReturn(pickle.name, "Expected pickle to have a name")} (example #${i + 1})`;
            createPickle(gherkinDocument, pickles, Object.assign(Object.assign({}, scenario), { name: exampleName }), pickle, testFilter);
        }
    }
    else {
        const scenarioId = assertions_1.assertAndReturn(scenario.id, "Expected scenario to have an id");
        const pickle = assertions_1.assertAndReturn(pickles.find((pickle) => pickle.astNodeIds && pickle.astNodeIds.includes(scenarioId)), `Expected to find a pickle associated with id = ${scenarioId}`);
        createPickle(gherkinDocument, pickles, scenario, pickle, testFilter);
    }
}
function createPickle(gherkinDocument, pickles, scenario, pickle, testFilter) {
    const astIdMap = gherkinDocumentsAstIdMaps.get(gherkinDocument);
    const tags = pickle.tags ? pickle.tags.map(mapTagName) : [];
    if (!testFilter.evaluate(tags)) {
        return;
    }
    it(scenario.name || "<unamed scenario>", function () {
        var _a, _b, _c, _d, _e;
        window.testState = {
            gherkinDocument,
            pickles,
            pickle,
        };
        registry_1.default.runBeforeHooks(this, tags);
        if (pickle.steps) {
            for (const pickleStep of pickle.steps) {
                const text = assertions_1.assertAndReturn(pickleStep.text, "Expected pickle step to have a text");
                const scenarioStep = assertions_1.assertAndReturn(astIdMap.get(assertions_1.assertAndReturn((_a = pickleStep.astNodeIds) === null || _a === void 0 ? void 0 : _a[0], "Expected to find at least one astNodeId")), `Expected to find scenario step associated with id = ${(_b = pickleStep.astNodeIds) === null || _b === void 0 ? void 0 : _b[0]}`);
                cy.wrap(null, { log: false }).then(() => {
                    Cypress.log({
                        name: "step",
                        displayName: assertions_1.assertAndReturn("keyword" in scenarioStep && scenarioStep.keyword, "Expected to find a keyword in the scenario step"),
                        message: text,
                    });
                });
                const argument = ((_c = pickleStep.argument) === null || _c === void 0 ? void 0 : _c.dataTable) ? new data_table_1.default(pickleStep.argument.dataTable)
                    : ((_e = (_d = pickleStep.argument) === null || _d === void 0 ? void 0 : _d.docString) === null || _e === void 0 ? void 0 : _e.content) ? pickleStep.argument.docString.content
                        : undefined;
                registry_1.default.runStepDefininition(this, text, argument);
            }
        }
        registry_1.default.runAfterHooks(this, tags);
    });
}
function collectTagNamesFromGherkinDocument(gherkinDocument) {
    const tagNames = [];
    for (const node of ast_helpers_1.traverseGherkinDocument(gherkinDocument)) {
        if ("tags" in node && node.tags) {
            for (const tag of node.tags) {
                tagNames.push(mapTagName(tag));
            }
        }
    }
    return tagNames;
}
function createTests(gherkinDocument, pickles) {
    const noopNode = { evaluate: () => true };
    const environmentTags = Cypress.env("TAGS");
    const testFilter = collectTagNamesFromGherkinDocument(gherkinDocument).includes("@focus")
        ? tag_expressions_1.default("@focus")
        : environmentTags
            ? tag_expressions_1.default(environmentTags)
            : noopNode;
    if (gherkinDocument.feature) {
        createFeature(gherkinDocument, pickles, gherkinDocument.feature, testFilter);
    }
}
exports.default = createTests;
